/**
 *#########################################################################
 *
 * A component of the Gatherer application, part of the Greenstone digital
 * library suite from the New Zealand Digital Library Project at the
 * University of Waikato, New Zealand.
 *
 * Author: John Thompson, Greenstone Digital Library, University of Waikato
 *
 * Copyright (C) 1999 New Zealand Digital Library Project
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *########################################################################
 */

package org.greenstone.gatherer;

import java.applet.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.net.*;
import javax.swing.JApplet;

import java.awt.*;
import java.io.*;
import java.util.*;
import javax.swing.*;
import org.greenstone.gatherer.Configuration;
import org.greenstone.gatherer.feedback.ActionRecorderDialog;
import org.greenstone.gatherer.gui.GUIManager;
import org.greenstone.gatherer.util.JarTools;
import org.greenstone.gatherer.util.Utility;
import org.greenstone.gatherer.shell.GShell;


public class GathererApplet extends JApplet implements ActionListener
{
    Gatherer gatherer;
    GUIManager g_man;
    Dimension size = new Dimension(800, 540);

    protected String fullLibraryURL(String address) 
    {
	String full_address = "";

        // make sure the URL has protocol, host, and file
        if (address.startsWith("http:")) {
            // the address has all the necessary components
	    full_address = address;
        } 
	else if (address.startsWith("/")) {
            // there is not protocol and host
            URL document = getDocumentBase();
	    int port_no = document.getPort();

            String port = (port_no>0) ? ":" + port_no : "";
            full_address = "http://" + document.getHost() + port  + address;
	}

	return full_address;
    }

    public void init()
    {
	try {
	    // Work-around to reduce number of 
	    // 'Could not lock user prefs' error messages.
	    // Thanks to Walter Schatz from the java forums.
	    System.setProperty("java.util.prefs.syncInterval","2000000");
	}

	catch (Exception exception) {
	    // convenient way of finding out if user has agreed to
	    // to requested security settings

	    JLabel lab 
		= new JLabel("Greenstone Librarian Interface Applet deactivated",
			     JLabel.CENTER);
	    getContentPane().add(lab);
	    return;
	}

	// Ensure platform specific LAF
	try {
	    UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
	}
	catch (Exception exception) {
	    exception.printStackTrace();
	}

	String gwcgi = getParameter("gwcgi");
	String library_cgi = fullLibraryURL(gwcgi);
	int cgi_base_cut = library_cgi.lastIndexOf('/');
	String cgi_base = library_cgi.substring(0,cgi_base_cut+1);

        String windows_home = getParameter("windowsHome");
        File gli_user_directory = Utility.setGLIUserFolder(windows_home);
	if (!gli_user_directory.exists()) {
	    if (!gli_user_directory.mkdirs()) {
		    System.err.println("Warning: Unable to make directory: " + gli_user_directory.toString());
	    }
	}

	gatherer = new Gatherer();

	String[] args = { "-gsdl", gli_user_directory.toString(),
			  "-library_url", library_cgi,
	                };

	GetOpt go = new GetOpt(args);

        String debug_param = getParameter("debug");

        if ((debug_param != null) && (debug_param.matches("true"))) {
            go.debug = true;
        }

	if (go.feedback_enabled) {
	    // If feedback is enabled, set up the recorder dialog.  
	    // Use the default locale for now - this will be changed by the
	    // Gatherer run method
	    Locale currLocale = Locale.getDefault(); 
	    ActionRecorderDialog dlg = new ActionRecorderDialog (currLocale);
	    gatherer.feedback_enabled = true;
	    gatherer.feedback_dialog = dlg;
	}

	Gatherer.isApplet = true;
	Gatherer.isGsdlRemote = true;
	Gatherer.cgiBase = cgi_base;

	Utility.BASE_DIR = go.gsdl_path;
	if (!Utility.BASE_DIR.endsWith(File.separator)) {
	    Utility.BASE_DIR += File.separator;
	}

	Utility.METADATA_DIR = Utility.BASE_DIR + "metadata" + File.separator;
	Utility.TMP_DIR = Utility.BASE_DIR + "tmp" + File.separator;

	Configuration.TEMPLATE_CONFIG_XML = "xml/configRemote.xml";
	Configuration.CONFIG_XML = "configRemote.xml";
	Configuration.GS3_CONFIG_XML = "config3Remote.xml";

	Gatherer.setCollectDirectoryPath(go.gsdl_path + "collect" + File.separator);
	File col_dir = new File(Gatherer.getCollectDirectoryPath());
	if (!col_dir.exists()) {

	    if (JarTools.isInJar(Utility.COLLECT_ZIP)) {
		// Dig out collect.zip from JAR file and unzip it (thereby
		// creating collect directory and descendants.
		//   This is done to seed gsdl home folder with config files
		//   for documented example collections so "base this on
		//   existing collection" works
		Utility.unzipFromJar(Utility.COLLECT_ZIP,gli_user_directory.toString());
	    }
	    else {
		// Prepare an empty collect dir for general use
		if (!col_dir.mkdir()) {
		    System.err.println("Warning: Unable to make directory: " + col_dir.toString());
		}
	    }
        }

	
        File tmp_dir = new File(Utility.TMP_DIR);
        if (!tmp_dir.exists()) {
            if (!tmp_dir.mkdir()) {
		System.err.println("Warning: Unable to make directory: " + tmp_dir.toString());
	    }

	    String username = System.getProperty("user.name");
	    if ((username != null) && (username != "")) {
		File user_tmp_dir = new File(Utility.TMP_DIR + File.separator + username);
		if (!user_tmp_dir.exists()) {
		    if (!user_tmp_dir.mkdir()) {
			System.err.println("Warning: Unable to make directory: " + user_tmp_dir.toString());
		    }
		}
	    }
        }

	File metadata_directory = new File(Utility.METADATA_DIR);
	if (!metadata_directory.exists()) {
	    // dig out metadata.zip from JAR file and unzip it
	    Utility.unzipFromJar(Utility.METADATA_ZIP, gli_user_directory.toString());
	}

	File plug_dat = new File(Utility.BASE_DIR + "plugins.dat");
	if (!plug_dat.exists()) {
	    JarTools.extractFromJar("plugins.dat", Utility.BASE_DIR, false);
	}

	File class_dat = new File(Utility.BASE_DIR + "classifiers.dat");
	if (!class_dat.exists()) {
	    JarTools.extractFromJar("classifiers.dat", Utility.BASE_DIR, false);
	}
	
	g_man = gatherer.init(size, go.gsdl_path, go.gsdl3_path, go.local_library_path,
				    go.library_url_string, go.debug, go.perl_path, 
				    go.no_load, go.filename, go.site_name, 
				    go.servlet_path, go.wget_path);

	JButton bttn = new JButton("Launch Greenstone Librarian Interface ...");
	bttn.addActionListener(this);
	getContentPane().add(bttn);
    }



    public void start() 
    {
	System.err.println("Start called");
    }



    public void stop()
    {
	System.err.println("Stop called");

    }

    public void destroy()
    {
	System.err.println("Destroy called");
	gatherer.exit();
	System.err.println("Done gatherer exit.");
    }

    public void actionPerformed(ActionEvent e)
    {
	gatherer.run(size, g_man);

	// If there was an open collection last session, reopen it.
	if (Gatherer.open_collection_file_path != null) {
	    Gatherer.c_man.loadCollection(Gatherer.open_collection_file_path);
	}
    }
}
